<?php

/**
 * Test: DIContainerBuilder.
 *
 * @author     David Grudl
 * @package    Nette\DI
 * @subpackage UnitTests
 */



require dirname(__FILE__) . '/../bootstrap.php';



class Service
{
	public $args;
	public $methods;

	static function create(IDIContainer $container)
	{
		$args = func_get_args();
		unset($args[0]);
		return new self($args);
	}

	function __construct()
	{
		$this->args = func_get_args();
	}

	function __call($nm, $args)
	{
		$this->methods[] = array($nm, $args);
	}

}



$container = new DIContainer;
$container->params['serviceClass'] = 'Service';
$container->params['arg1'] = 'a';
$container->params['tag'] = 'attrs';

$builder = new DIContainerBuilder;

$builder->addDefinitions($container, array(
	'one' => array(
		'class' => '%serviceClass%',
		'arguments' => array(
			'%arg1%', 'b',
		),
		'methods' => array(
			array('methodA', array('%arg1%', 'b')),
		),
		'tags' => array(
			'panel' => '%tag%',
		)
	),
	'two' => array(
		'factory' => '%serviceClass%::create',
		'arguments' => array(
			'%arg1%', '@one',
		),
	),
	'three' => array(
		'factory' => array('%serviceClass%', 'create'),
	),
	'four' => '@three',
));

Assert::true( $container->getService('one') instanceof Service );
Assert::same( array('a', 'b'), $container->getService('one')->args );
Assert::same( array(array('methodA', array('a', 'b'))), $container->getService('one')->methods );
Assert::same( array('one' => array('attrs')), $container->getServiceNamesByTag('panel') );

Assert::true( $container->getService('two') instanceof Service );
Assert::equal( array(array(1 => 'a', $container->getService('one'))), $container->getService('two')->args );

Assert::true( $container->getService('three') instanceof Service );
Assert::true( $container->getService('four') instanceof Service );


$builder->addDefinitions($container, array(
	'bad' => array(
		'class' => '%missing%',
	)
));
Assert::throws(create_function('', 'extract(NCFix::$vars['.NCFix::uses(array('container'=>$container)).'], EXTR_REFS);
	$container->getService(\'bad\');
'), 'InvalidArgumentException', "Missing item 'missing'.");
